local Entity = class("Entity")

function Entity:initialize(t)
	for k,v in pairs(t) do
		self[k] = v
	end
	if self.id == nil then self.id = self.name.."-"..os.time().."-"..random() end
	if self.inventory then self.inventory = {} end
	self.status = {}
	self.passives = {}
	self.AP = 10
	self.currentHp = self.maxHp
	self.start_of_turn = true
end

function Entity:update(map)
	if self.start_of_turn == true then
		self:startTurn(map)
		self.start_of_turn = false
	end
	if self.AP > 0 then
		self:action(map)
	end
	if self.AP <= 0 then
		self:endTurn(map)
		self.start_of_turn = true
		self.AP = 10
		return true
	end
end

function Entity:startTurn(map)
	if self.inventory then
		for i,item in ipairs(self.inventory) do
			if item.update then item:update() end
		end
	end
	if self.equipment then
		for i,item in ipairs(self.equipment) do
			if item.update then item:update() end
		end
	end
	for i in pairs(self.status) do
		self:applyStatus(i)
		self.status[i] = self.status[i] - 1
		if self.status[i] == 0 then self.status[i] = nil end
	end
end

function Entity:applyStatus(status)
	if status == "stunned" then self.AP = -1000
	elseif status == "energized" then self.AP = self.AP + 5
	end
end

function Entity:action(map)
	
end

function Entity:endTurn(map)

end

function Entity:interact(tx,ty,cx,cy,map)
	local cx,cy = cx,cy
	if not cx then cx,cy = map:findEntity(self.id) end
	if tx == cx and ty == cy then
		if self.inventory and map[cx][cy].items then
			local item = map:removeItem(cx,cy)
			self:addToInventory(item)
		else
			self:rest()
		end
	end
	if tx ~= cx and ty ~= cy then
		if map:collision(tx,cy) or map:collision(cx,ty) then return end
	end
	if tx < 1 or tx > map.WIDTH or ty < 1 or ty > map.HEIGHT then return end
	local cell = map[tx][ty]
	if cell.entity then
		if cell.entity.name == self.name then self:rest()
		else self:attackEntity(cell.entity,map)
		end
	elseif cell.prop then
	else
		if cell.terrain == "#" then 
			self:tapWall(map,tx,ty)
		elseif cell.terrain == "." then
			map:moveEntity(cx,cy,tx,ty)
			self.AP = self.AP - self:moveCost()
		end
	end
end

function Entity:rest()
	self.AP = self.AP - 10
end

function Entity:addToInventory(item)
	if self.inventory and #self.inventory < self.inventory_limit then
		table.insert(self.inventory,item)
		self.AP = self.AP - 10
		return true
	end
end

function Entity:removeFromInventory(i)
	if self.inventory and self.inventory[i] then
		return table.remove(self.inventory,i)
	end
end

function Entity:drop(item,map)
	local cx,cy = map:findEntity(self.id)
	map:insertItem(item,cx,cy)
	return item,cx,cy
end

function Entity:equipItem(item)
	if item and self.equipment and #self.equipment < self.equipment_limit then
		table.insert(self.equipment,item)
		self.AP = self.AP - 10
	end
	return item
end

function Entity:unequipItem(i)
	if self.equipment and self.equipment[i] then
		local item = table.remove(self.equipment,i)
		if not self:addToInventory(item) then
		
		end
	end
	return item
end

function Entity:tapWall(map,tx,ty)
	table.insert(map.animations,WallBumpAnimation:new(tx,ty))
	self.AP = self.AP - 5
end

function Entity:moveCost()
	if self.runner then return 5
	elseif self.status.hasted then return 5
	else return 10
	end
end

function Entity:attackCost()
	if self.quickstrike then return 5
	elseif self.status.hasted then return 5
	else return 10
	end
end

function Entity:attackDamage()
	local total_attack = self.attack
	if self.equipment and #self.equipment > 0 then
		for i,item in ipairs(self.equipment) do
			total_attack = total_attack + item:getAttack()
		end
	end
	return total_attack
end

function Entity:attackEntity(enemy,map)
	local attack_hit,damage_dealt,enemy_is_dead = false, 0, false
	states[gamestate].currentLog:add("%s attacks %s.",self.name,enemy.name)
	if enemy:beingAttacked() then
		attack_hit = true
		damage_dealt,enemy_is_dead = enemy:takeDamage(self:attackDamage(),"physical",self.name,map)
	else
		states[gamestate].currentLog:add("The attack misses.")
	end
	self.AP = self.AP - self:attackCost()
	return attackHit,damage_dealt,enemy_is_dead
end

function Entity:beingAttacked()
	local evasion = 0
	if self.evasion then evasion = evasion + self.evasion end
	if math.random(100) > evasion then return true else return false end
end

function Entity:takeDamage(damage,damage_type,source,map)
	local damage_dealt,is_dead = 0,false
	self.currentHp = self.currentHp - damage
	local cx,cy = map:findEntity(self.id)
	table.insert(map.animations,HitAnimation:new(cx,cy))
	if self.currentHp <= 0 then
		self.currentHp = 0
		self:death(map)
		is_dead = true
	end
	damage_dealt = damage
	return damage_dealt,is_dead
end

function Entity:heal(healing)
	self.currentHp = self.currentHp + healing
	if self.currentHp > self.maxHp then self.currentHp = self.maxHp end
end

function Entity:death(map)
	self.AP = 0
	local map = map
	local cx,cy = map:findEntity(self.id)
	map:destroyEntity(cx,cy)
	states[gamestate].currentLog:add("%s dies.",self.name)
	return cx,cy
end

function Entity:getGraphics()
	local qx,qy,r,g,b,br,bg,bb
	qx,qy = self.graphics.qx, self.graphics.qy
	if #self.status == 0 then
		r,g,b = self.graphics.r, self.graphics.g, self.graphics.b
	end
	return qx,qy,r,g,b,br,bg,bb
end

Entity.pathfinding = require "pathfinding"

return Entity