local Hero = class("Hero",Entity)

local hero_property = {
	name = "the hero",
	id = "hero",
	graphics = {qx = 1, qy = 5},
	attack = 10,
	maxHp = 100,
	inventory = true,
	inventory_limit = 30,
	equipment = {},
	equipment_limit = 3
	}

function Hero:initialize()
	Entity.initialize(self, hero_property)
	self.nextMove = false
end

function Hero:action(map)
	local cx,cy = map:findEntity(self.id)
	if self.nextMove then
		self:interact(cx+self.nextMove.dx,cy+self.nextMove.dy,cx,cy,map)
		self.nextMove = false
	end
end

function Hero:attackEntity(enemy,map)
	enemy.status.stunned = 5
	Entity.attackEntity(self,enemy,map)
end

function Hero:keyInput(key,map)
	if self.nextMove == false and self.start_of_turn == false then
		if key == "up" or key == "kp8" then self.nextMove = {dx = 0, dy = -1}
		elseif key == "down" or key == "kp2" then self.nextMove = {dx = 0, dy = 1}
		elseif key == "left" or key == "kp4" then self.nextMove = {dx = -1, dy = 0}
		elseif key == "right" or key == "kp6" then self.nextMove = {dx = 1, dy = 0}
		elseif key == "kp7" then self.nextMove = {dx = -1, dy = -1}
		elseif key == "kp9" then self.nextMove = {dx = 1, dy = -1}
		elseif key == "kp3" then self.nextMove = {dx = 1, dy = 1}
		elseif key == "kp1" then self.nextMove = {dx = -1, dy = 1}
		elseif key == " " or key == "kp5" then self.nextMove = {dx = 0, dy = 0}
		elseif key == "e" then
			self:equipItem(self:removeFromInventory(1))
		elseif key == "u" then
			self:unequipItem(1)
		elseif key == "d" then
			local item = self:removeFromInventory(1)
			if item then self:drop(item) end
		elseif key == "z" then
			self:rest()
		elseif key == "1" then
			if self.equipment[1] and self.equipment[1].active then self.equipment[1]:active(self,map) end
		elseif key == "2" then
			if self.equipment[2] and self.equipment[2].active then self.equipment[2]:active(self,map) end
		elseif key == "3" then
			if self.equipment[3] and self.equipment[3].active then self.equipment[3]:active(self,map) end
		elseif key == "4" then
			if self.equipment[4] and self.equipment[4].active then self.equipment[4]:active(self,map) end
		elseif key == "5" then
			if self.equipment[5] and self.equipment[5].active then self.equipment[5]:active(self,map) end
		elseif key == "6" then
			if self.equipment[6] and self.equipment[6].active then self.equipment[6]:active(self,map) end
		elseif key == "q" then
			if self.inventory[1] and self.inventory[1].consume then self.inventory[1]:consume(self,1) end
		end
	end
end

function Hero:death(map)
	states[gamestate].currentLog:add("You die...")
	local cx,cy = map:findEntity(self.id)
	table.insert(map.animations,Explosion:new(self,cx,cy,100,255,0,0,10))
end

function Hero:getGraphics()
	local qx,qy,r,g,b,br,bg,bb,a = Entity.getGraphics(self)
	local relativeHp = self.currentHp/self.maxHp
	r,g,b = math.floor(2550*relativeHp),math.floor(255*relativeHp),math.floor(255*relativeHp)
	return qx,qy,r,g,b,br,bg,bb,a
end

function Hero:UI()
	love.graphics.print(string.format("HP: %d/%d",self.currentHp,self.maxHp),0,0)
	love.graphics.print(string.format("Equipment: "),0,20)
	for i, item in ipairs(self.equipment) do
		love.graphics.print(capitalize(item.name),10,20+10*i)
	end
end

return Hero