local pathfinding = function(entity,map,sx,sy,tx,ty)
	local distBetween = distBetween
	local closed_set = {}
	local open_set = {{x=sx,y=sy}}
	
	local came_from = {}
	local g_score = {}
	local f_score = {}
	local cost_map = {}
	for x = 1, map.WIDTH do
		came_from[x] = {}
		g_score[x] = {}
		f_score[x] = {}
		cost_map[x] = {}
		for y = 1, map.HEIGHT do
			came_from[x][y] = false
			g_score[x][y] = math.huge
			f_score[x][y] = math.huge
			
			local cell = map[x][y]
			local cost = 1
			if cell.entity then
				cost = 10
			elseif cell.prop and cell.prop.impassable then cost = false
			elseif cell.terrain == "#" then cost = false
			end
			cost_map[x][y] = cost
		end
	end
	g_score[sx][sy] = 0
	f_score[sx][sy] = distBetween(sx,sy,tx,ty)
	cost_map[tx][ty] = 0
	local function isClosed(x,y)
		for i,v in ipairs(closed_set) do
			if v.x == x and v.y == y then return true end
		end
	end
	local function isOpen(x,y)
		for i,v in ipairs(open_set) do
			if v.x == x and v.y == y then return true end
		end
	end
	
	local loops = 0
	while #open_set > 0 and loops < 500 do
		loops = loops + 1
		table.sort(open_set, function(a,b)
			if f_score[a.x][a.y] < f_score[b.x][b.y] then return true end
		end)
		local current = table.remove(open_set,1)
		local cx,cy = current.x, current.y
		if current.x == tx and current.y == ty then
			local total_path = { current }
			while came_from[current.x][current.y] do
				current = came_from[current.x][current.y]
				table.insert(total_path,1,current)
			end
			table.remove(total_path,1)
			return total_path
		end
		table.insert(closed_set,current)
		local neighbors = {}
		if cx > 1 and cost_map[cx-1][cy] then
			table.insert(neighbors,{x=cx-1,y=cy})
			if cost_map[cx][cy-1] and cost_map[cx-1][cy-1] then table.insert(neighbors,{x=cx-1,y=cy-1}) end
			if cost_map[cx][cy+1] and cost_map[cx-1][cy+1] then  table.insert(neighbors,{x=cx-1,y=cy+1}) end
		end
		if cx < map.WIDTH and cost_map[cx+1][cy] then
			table.insert(neighbors,{x=cx+1,y=cy})
			if cost_map[cx][cy-1] and cost_map[cx+1][cy-1] then table.insert(neighbors,{x=cx+1,y=cy-1}) end
			if cost_map[cx][cy+1] and cost_map[cx+1][cy+1] then table.insert(neighbors,{x=cx+1,y=cy+1}) end
		end
		if cost_map[cx][cy-1] then table.insert(neighbors,{x=cx,y=cy-1}) end
		if cost_map[cx][cy+1] then table.insert(neighbors,{x=cx,y=cy+1}) end
		for _,neighbor in ipairs(neighbors) do
			if not isClosed(neighbor.x,neighbor.y) then
				local tentative_g_score = g_score[cx][cy] + cost_map[neighbor.x][neighbor.y]
				if not isOpen(neighbor.x,neighbor.y) or tentative_g_score < g_score[neighbor.x][neighbor.y] then
					came_from[neighbor.x][neighbor.y] = {x = cx, y = cy}
					g_score[neighbor.x][neighbor.y] = tentative_g_score
					f_score[neighbor.x][neighbor.y] = g_score[neighbor.x][neighbor.y] + distBetween(neighbor.x, neighbor.y, tx, ty)
					if not isOpen(neighbor.x,neighbor.y) then
						table.insert(open_set, {x = neighbor.x, y = neighbor.y})
					end
				end
			end
		end
	end
	return false
end

return pathfinding